#include "precompile.h"
#include <mutex>
#include "ordermgr.h"
#include <a8/mysql.h>
#include <a8/udplog.h>
#include "jsondatamgr.h"
#include <unistd.h>
#include "framework/cpp/utils.h"
#include "framework/cpp/httpclientpool.h"
#include <a8/mutable_xobject.h>

void OrderMgr::Init()
{
    sub_orderid_ = 1;
    last_gen_order_time_ = g_nowtime;
    last_ping_db_tick_ = a8::XGetTickCount();
    //assert(mysql_conn_.Connect("127.0.0.1", 3306, "root", "keji178", "paydb"));
    mysql_conn_ = new a8::mysql::Connection();
    query_ = mysql_conn_->CreateQuery();
    assert(mysql_conn_->Connect(JsonDataMgr::Instance()->GetMysqlConf()->At("host")->AsXValue(),
                                JsonDataMgr::Instance()->GetMysqlConf()->At("port")->AsXValue(),
                                JsonDataMgr::Instance()->GetMysqlConf()->At("user")->AsXValue(),
                                JsonDataMgr::Instance()->GetMysqlConf()->At("passwd")->AsXValue(),
                                "paydb"));
    f8::InitMysqlConnection(query_);
    LoadPedingOrders();
}

void OrderMgr::UnInit()
{
}

void OrderMgr::Update()
{
    if (a8::XGetTickCount() - last_ping_db_tick_ > 1000 * 60 * 5) {
        last_ping_db_tick_ = a8::XGetTickCount();
        if (query_->ExecQuery("SELECT 1;", {}) <= 0) {
            a8::UdpLog::Instance()->Warning("mysql disconnect", {});
            if (mysql_conn_->Connect(JsonDataMgr::Instance()->GetMysqlConf()->At("host")->AsXValue(),
                                     JsonDataMgr::Instance()->GetMysqlConf()->At("port")->AsXValue(),
                                     JsonDataMgr::Instance()->GetMysqlConf()->At("user")->AsXValue(),
                                     JsonDataMgr::Instance()->GetMysqlConf()->At("passwd")->AsXValue(),
                                     "paydb")) {
                f8::InitMysqlConnection(query_);
                a8::UdpLog::Instance()->Info("mysql reconnect successed", {});
            } else {
                a8::UdpLog::Instance()->Info("mysql reconnect failed", {});
            }
        }
    }
    UpdatePedingOrders();
}

void OrderMgr::UpdatePedingOrders()
{
    a8::UdpLog::Instance()->Debug("UpdatePedingOrders peding_orders_ size:%d", { peding_orders_.size() });
    for (std::map<std::string, OrderInfo>::iterator itr = peding_orders_.begin();
         itr != peding_orders_.end(); ++itr){
        if (g_nowtime - itr->second.lastchecktime > 15) {
            itr->second.lastchecktime = g_nowtime;
            OrderInfo info = itr->second;
            SendGMMsg(
                      info.serverid,
                      info.orderid,
                      info.roleid,
                      info.itemid,
                      info.price,
                      info.rolename
                      );
            a8::UdpLog::Instance()->Debug("UpdatePedingOrders orderid:%s roleid:%d name:%s serverid:%d", { info.orderid,info.roleid,info.rolename,info.serverid });
        }
    }
}

//支付成功并且gameserver没有确认的订单
void OrderMgr::LoadPedingOrders()
{
    int ret = query_->ExecQuery("SELECT orderid, itemid, roleid, rolename, price, serverid, accountid "
                               "FROM orderinfo WHERE sp_pay_result=1 AND status=0;", {});
    if (ret < 0) {
        return;
    }
    while (!query_->Eof()) {
        OrderInfo info;
        info.orderid = query_->GetValue(0).GetString();
        info.itemid = query_->GetValue(1);
        info.roleid = query_->GetValue(2).GetString();
        info.rolename = query_->GetValue(3).GetString();
        info.price = query_->GetValue(4);
        info.lastchecktime = g_nowtime - 60 * 1;
        info.serverid = query_->GetValue(5);
        info.accountid = query_->GetValue(6).GetString();
        peding_orders_[info.orderid] = info;
        query_->Next();
    }
}

std::string OrderMgr::GenOrderId(int itemid, int price)
{
    char orderid[80] = {0};
    {
        time_t nowtime = time(NULL);
        if (nowtime == last_gen_order_time_) {
            sub_orderid_++;
            if (sub_orderid_ >= 10000) {
                while (nowtime <= last_gen_order_time_) {
                    ::usleep(100 * 10000);
                    nowtime = time(NULL);
                }
                sub_orderid_ = 1;
            }
        } else {
            sub_orderid_ = 1;
        }
        last_gen_order_time_ = nowtime;

        struct tm tm_nowtime = {0};
        struct tm *ptr = localtime_r(&nowtime, &tm_nowtime);
        char strtime[80] = {0};
        strftime(strtime, 80, "%y%m%d%H%M%S", ptr);

        sprintf(orderid, "%s%5d", strtime, sub_orderid_);
        for (int i = 0; i < a8::ArraySize(orderid); i++) {
            if (orderid[i] == ' ') {
                orderid[i] = '0';
            }
        }
    }

    std::string strorderid;
    strorderid = a8::Format("%d_%d_%s",
                            {
                             itemid,
                             price,
                             std::string(orderid)
                            }
                            );
    return strorderid;
}

//new
bool OrderMgr::AddNewOrder(const std::string& accountid,
                           int serverid,
                           const std::string& roleid,
                           const std::string& rolename,
                           int itemid, int price,
                           const std::string& sp_orderid,
                           OrderInfo& orderinfo)
{
    std::string orderid = GenOrderId(itemid, price);
    //int serverid = f8::ExtractGameIdFromAccountId(accountid);
    int channel = f8::ExtractChannelIdFromAccountId(accountid);

    orderinfo.orderid = orderid;
    orderinfo.serverid = serverid;
    orderinfo.roleid = roleid;
    orderinfo.rolename = rolename;
    orderinfo.channel = a8::XValue(channel).GetString();
    orderinfo.itemid = itemid;
    orderinfo.price = price;
    orderinfo.accountid = accountid;
    orderinfo.sp_orderid = sp_orderid;

    return query_->ExecScript("INSERT INTO orderinfo(orderid, serverid, roleid, rolename, channel, itemid, price, status, createtime, accountid, sp_orderid)" \
                              "VALUES('%s', %d, '%s', '%s', '%s', '%s', %d, %d, %d, '%s','%s');",
                              {
                               orderinfo.orderid,
                               orderinfo.serverid,
                               orderinfo.roleid,
                               orderinfo.rolename,
                               orderinfo.channel,
                               orderinfo.itemid,
                               orderinfo.price,
                               0,
                               g_nowtime,
                               orderinfo.accountid,
                               orderinfo.sp_orderid
                              });
}

void OrderMgr::AddNewPayNotify(
                    int pay_type,
                    const std::string& orderid,
                    int payresult,
                    int& result)
{
    switch (pay_type) {
    case 1:
        result = ProcOrderNotifyNew(pay_type, orderid, payresult);
        break;
    case 2:
        #if 0
        result = ProcIosPayOrderNotify(delnode);
        #endif
        break;
    case 3:
        #if 0
        result = ContinueSubscriptions(delnode);
        #endif
        break;
    default:
        result = 5;
    }
}

void OrderMgr::G2PPayNotifyACK(a8::XParams& param, a8::XObject& data)
{
    int errcode = data.Get("errcode").GetInt();
    if (errcode == 0) {
        std::string orderid = param.sender;
        if (peding_orders_.find(orderid) != peding_orders_.end()) {
            if (query_->ExecScript("UPDATE orderinfo SET status=%d, confirmtime=%d WHERE orderid='%s';",
                                   {
                                    1,
                                    g_nowtime,
                                    orderid
                                   })) {
                peding_orders_.erase(orderid);
            } else {
                a8::UdpLog::Instance()->Warning("update orderinfo error  %s", {query_->GetError()});
            }
        }
    } else {
        a8::UdpLog::Instance()->Warning("G2PPayNotifyACK resp error %s",
                                        {
                                         data.Get("errmsg").GetString()
                                        });
    }
}

int OrderMgr::ProcOrderNotifyNew(int pay_type,
                                 const std::string& orderid,
                                 int payresult)
{
    if (payresult != 0 && payresult != 1) {
        return 4;
    }

    int ret = query_->ExecQuery("SELECT orderid, itemid, roleid, rolename, price, sp_pay_result, serverid "
                                "FROM orderinfo WHERE orderid='%s' and sp_confirm_time=0;",
                                { orderid});
    if (ret > 0) {
        if (query_->GetValue(5).GetInt() == 0) {
            OrderInfo info;
            info.orderid = query_->GetValue(0).GetString();
            info.itemid = query_->GetValue(1);
            info.roleid = query_->GetValue(2).GetString();
            info.rolename = query_->GetValue(3).GetString();
            info.price = query_->GetValue(4);
            info.serverid = query_->GetValue(6);
            if (query_->ExecScript("UPDATE orderinfo SET sp_pay_result=%d, sp_confirm_time=%d WHERE orderid='%s';",
                                   {
                                    payresult,
                                    g_nowtime,
                                    orderid
                                   })) {
                if (payresult == 1) {
                    //http 异步，加入队列主线程循环执行
                    peding_orders_[info.orderid] = info;
                }
                return 0;
            } else {
                return 2;
            }
        } else {
            return 3;
        }
    } else {
        return 1;
    }
}

void OrderMgr::SendGMMsg(int serverid,
                         const std::string& orderid,
                         const std::string& roleid,
                         int itemid, int price,
                         const std::string& rolename)
{
    std::string url;
    {
        std::string ip = JsonDataMgr::Instance()->GetTargetServerClusterConf(serverid)->At("ip")->AsXValue();
        std::string port = JsonDataMgr::Instance()->GetTargetServerClusterConf(serverid)->At("port")->AsXValue();
        url = "http://" + ip + ":" + port + "/webapp/index.php?c=Pay&a=payNotify";
        url = url + "&orderid=" + orderid
            + "&roleid=" + roleid
            + "&itemid=" + a8::XValue(itemid).GetString()
            + "&price=" + a8::XValue(price).GetString()
            + "&name=" + rolename;
    }

    a8::MutableXObject* params_xobj = a8::MutableXObject::NewObject();
    {

    }

    auto on_ok = [] (a8::XParams& param, a8::XObject& data)
                 {
                     OrderMgr::Instance()->G2PPayNotifyACK(param, data);
                 };
    auto on_error = [] (a8::XParams& param, const std::string& response)
                    {
                        a8::UdpLog::Instance()->Warning("http gameserver error %s", {response});
                    };
    f8::HttpClientPool::Instance()->HttpGet(
                                            a8::XParams()
                                            .SetSender(orderid)
                                            ,
                                            on_ok,
                                            on_error,
                                            url.c_str(),
                                            *params_xobj,
                                            rand()
                                            );
    delete params_xobj;
}

void OrderMgr::ExecSql(f8::JsonHttpRequest* request)
{
    int ret = query_->ExecQuery(request->request.Get("sql").GetString().c_str(), {});
    if (ret >= 0) {
        request->resp_xobj->SetVal("errcode", 0);
        request->resp_xobj->SetVal("errmsg", "");
        a8::MutableXObject* dataset = a8::MutableXObject::NewArray();
        while (!query_->Eof()) {
            a8::MutableXObject* row = a8::MutableXObject::NewArray();
            for (int i = 0; i < query_->FieldsNum(); ++i) {
                row->Push(query_->GetValue(i));
            }
            dataset->Push(*row);
            delete row;
            query_->Next();
        }
        request->resp_xobj->SetVal("dataset", *dataset);
        delete dataset;
    } else {
        request->resp_xobj->SetVal("errcode", 100);
        request->resp_xobj->SetVal("errmsg", query_->GetError());
    }
}
