#pragma once

namespace a8
{
    namespace mysql
    {
        class Query;
        class Connection;
    }
}

struct OrderInfo
{
    std::string seqid;
    std::string orderid;
    std::string accountid;
    int itemid = 0;
    int price = 0;
    int customprice = 0;
    int serverid = 0;
    std::string roleid;
    std::string rolename;
    std::string sp_accountid;
    std::string channel;
    std::string itemname;
    std::string itemmemo;
    std::string sp_orderid;
    time_t lastchecktime = 0;
};

struct AddOrderResult
{
    bool isok = false;
    OrderInfo orderinfo;
};

struct AddOrderNode;
struct PayNotifyNode;
struct ReissueNode;
class OrderMgr : public a8::Singleton<OrderMgr>
{

 private:
    OrderMgr() {};
    friend class a8::Singleton<OrderMgr>;

 public:
    void Init();
    void UnInit();

    void Update();
    bool AddNewOrder(const std::string& accountid,
                     int serverid,
                     const std::string& roleid,
                     const std::string& rolename,
                     int itemid, int price,
                     const std::string& sp_orderid,
                     OrderInfo& orderinfo);
    void AddNewPayNotify(
                        int pay_type,
                        const std::string& orderid,
                        int payresult,
                        int& result);
    void G2PPayNotifyACK(a8::XParams& param, a8::XObject& data);

    void ExecSql(f8::JsonHttpRequest* request);

 private:
    void UpdatePedingOrders();
    void LoadPedingOrders();
    std::string GenOrderId(int itemid, int price = 0);
    int ProcOrderNotifyNew(int pay_type,
                           const std::string& orderid,
                           int payresult);
    void SendGMMsg(int serverid,
                   const std::string& orderid,
                   const std::string& roleid,
                   int itemid, int price,
                   const std::string& rolename);

 private:
    a8::mysql::Connection* mysql_conn_ = nullptr;
    a8::mysql::Query* query_ = nullptr;
    a8::tick_t last_ping_db_tick_ = 0;

    int last_gen_order_time_ = 0;
    int sub_orderid_ = 0;

    std::mutex *pay_notify_result_mutex_;
    std::map<int, int> pay_notify_results_;

    std::map<std::string, OrderInfo> peding_orders_;
};
